#!/usr/bin/env wish

#******************************
# 
# osSmartDB 
# Version 2016-08-10
# Copyright (C) 2016 Oliver Scholl < tcltk@posteo.de > < www.tcltk.info >
#
# Requirement:
# tcl 8.5 or higher
# tk 8.5 or higher
# sqlite3
#
# Recommendation: none
#
# Description of the program:
# This is a database program. Please first create a directory to store the database.
# Place into this directory two text files:
#
# Tabelle.txt: 
# Write in each row the technical name of the database field. It must not be "ID".
# If you want you can add a whitespace and a label (in quotation marks).
# If the first character of the database fieldname is a number between 1 and 9,
# it determines the height of the field in the userinterface.
# Example:
# forename "Forename:"
# lastname "Last Name:"
# zip "Zip Code:"
# 4comment "Comment:"
# 
# Liste.txt:
# Write in each row the database fieldname that should be shown in the listbox. It determines the order, too.
# If you want you can add a whitespace and a seperator (in quotation marks).
# Example:
# lastname ", "
# forename
#
# License:
# This program is free software; you can redistribute it and/or 
# modify it under the terms of the GNU General Public License as 
# published by the Free Software Foundation; either version 2 of 
# the license, or any later version.
#
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# merchantability or fitness for a particular purpose. 
# See the GNU General Public License for mor details.
#
#*************************************

package require sqlite3
package require msgcat

dict set Glob ProgrammOrdner ".osProgramme"
dict set Glob ProgrammName "osSmartDB"
dict set Glob ProgrammVersion "2016-08-10"
dict set Glob ProgrammTitel "SmartDB"

proc KonfigurationLaden1 {} {
	# lädt die Konfigurationsdaten (Teil1)
	global Glob

	set tmp [dict get $Glob KonfigurationDateiname]
	if {[glob -nocomplain -types f $tmp] eq ""} {return}
	set f [open $tmp r]
	dict set Glob Sprache [gets $f]
	dict set Glob DatenbankOrdner [gets $f]
	close $f
}

proc KonfigurationLaden2 {} {
	# lädt die Konfigurationsdaten (Teil2)
	global Glob

	set tmp [dict get $Glob KonfigurationFenster]
	if {[glob -nocomplain -types f $tmp] eq ""} {return}
	set f [open $tmp r]
	dict set Glob Fensterbreite [gets $f]
	dict set Glob Fensterhoehe [gets $f]
	dict set Glob FensterX [gets $f]
	dict set Glob FensterY [gets $f]
	close $f
}

proc KonfigurationSpeichern {} {
	# speichert die Konfigurationsdaten
	global Glob

	set tmp [dict get $Glob KonfigurationDateiname]
	set f [open $tmp w]
	puts $f [dict get $Glob Sprache]
	puts $f [dict get $Glob DatenbankOrdner]
	close $f

	set tmp [dict get $Glob KonfigurationFenster]
	set f [open $tmp w]
	puts $f [winfo width .]
	puts $f [winfo height .]
	puts $f [winfo x .]
	puts $f [winfo y .]
	close $f
}

proc FelderEinlesen {} {
	# liest die Tabelle.txt ein und erstellt eine Liste mit den Feldern und eine Liste mit den Labels
	global Glob

	set ListeFelder ""
	set ListeLabels ""
	set ListboxFelder ""
	set ListboxTrennzeichen ""
	set ListeHoehe ""
	
	set tmp [dict get $Glob Tabelle]
	set f [open $tmp r]
	while {[gets $f Zeile] >= 0} {
		set Elemente [split $Zeile]
		set Element [lindex $Elemente 0]
		set ErstesZeichen [string index $Element 0]
		if {$ErstesZeichen >= "1" && $ErstesZeichen <= "9"} {
			set Element [string range $Element 1 end]
			lappend ListeHoehe $ErstesZeichen
		} else {
			lappend ListeHoehe 1
		}
		lappend ListeFelder $Element

		set Elemente [split $Zeile "\""]
		lappend ListeLabels [lindex $Elemente 1]
	}
	close $f

	set GlobListboxAusgabe ""
	set tmp [dict get $Glob ListboxAusgabe]
	set f [open $tmp r]
	while {[gets $f Zeile] >= 0} {
		set Elemente [split $Zeile]
		lappend ListboxFelder [lindex $Elemente 0]

		set Elemente [split $Zeile "\""]
		set Trennzeichen [lindex $Elemente 1]
		if {$Trennzeichen eq ""} {
			set Trennzeichen ", "
		}
		lappend ListboxTrennzeichen $Trennzeichen
	}
	close $f
	# letztes Trennzeichen löschen
	set ListboxTrennzeichen [lreplace $ListboxTrennzeichen end end ""]
	
	# Rückgaben
	dict set Glob ListeFelder $ListeFelder
	dict set Glob ListeLabels $ListeLabels
	dict set Glob ListeHoehe $ListeHoehe
	dict set Glob ListboxFelder $ListboxFelder
	dict set Glob ListboxTrennzeichen $ListboxTrennzeichen
}

proc DatenbankErstellen {} {
	# erstellt die Datenbank
	global Glob
	
	set ListeFelder [dict get $Glob ListeFelder]
	set Befehl "ID integer primary key autoincrement,"
	foreach Element $ListeFelder {
		append Befehl "$Element text,"
	}
	set Befehl [string range $Befehl 0 end-1]
	db1 eval "create table Tabelle ($Befehl)"
}

proc DatenbankOeffnen {} {
	global Glob
	
	set Datenbank [dict get $Glob Datenbank]
	# öffnet die Datenbank. Wenn die Datenbank noch nicht existiert, wird die Datenbank erstellt
	if {[glob -nocomplain -types f $Datenbank] eq ""} {
		# Datenbank existiert nicht
		sqlite3 db1 $Datenbank
		DatenbankErstellen
	} else {
		# Datenbank existiert
		sqlite3 db1 $Datenbank ; # Alternativ: sqlite3 db1 ":memory:"
	}
}

proc DatenbankSchliessen {} {
	# schließt die Datenbank
	catch db1 close
}

proc DatensatzAnzeigen {ID} {
	# ermittelt aus der Datenbank die Daten zu einem bestimmten Datensatz
	global Glob
	global GlobEntryInhalt
	
	set ListeFelder [dict get $Glob ListeFelder]
	
	set Befehl "select ID,"
	foreach Feld $ListeFelder {
		append Befehl "$Feld,"
	}
	set Befehl [string range $Befehl 0 end-1]
	append Befehl " from Tabelle where ID=$ID"
	db1 eval "$Befehl" Ergebnis {
		#parray Ergebnis

		set ListeHoehe [dict get $Glob ListeHoehe]
		set GlobEntryInhalt(ID) $Ergebnis(ID)
		for {set Zaehler 0} {$Zaehler < [llength $ListeFelder]} {incr Zaehler} {
			set Hoehe [lindex $ListeHoehe $Zaehler]							
			set Feld [lindex $ListeFelder $Zaehler]
			set GlobEntryInhalt($Zaehler) $Ergebnis($Feld)
			if {$Hoehe > 1} {
				set Status [.frRechts.c.frFelder.tx$Zaehler cget -state]
				.frRechts.c.frFelder.tx$Zaehler configure -state normal
				.frRechts.c.frFelder.tx$Zaehler delete 1.0 end
				.frRechts.c.frFelder.tx$Zaehler insert end $GlobEntryInhalt($Zaehler)
				.frRechts.c.frFelder.tx$Zaehler configure -state $Status
			}
		}
	}
}

proc Suchen {Suchbegriff} {
	# sucht in der Datenbank nach dem Suchbegriff
	# Rückgabe: Dictionary mit den IDs und den Einträgen für die Listbox
	global Glob
	
	set ListeFelder [dict get $Glob ListeFelder]
	set ListboxFelder [dict get $Glob ListboxFelder]
	set ListboxTrennzeichen [dict get $Glob ListboxTrennzeichen]
	set ID ""
	
	if {[string tolower [string range $Suchbegriff 0 1]] == "id" && [string index $Suchbegriff 2] eq "="} {
		set Elemente [split $Suchbegriff "="]
		set ID [lindex $Elemente 1]
	}
	
	set Suche "%"
	append Suche $Suchbegriff
	append Suche "%"

	set ListeEintrag ""
	set ListeID ""
	
	set Befehl "select ID,"
	foreach Feld $ListboxFelder {
		append Befehl "$Feld,"
	}
	set Befehl [string range $Befehl 0 end-1]
	append Befehl " from Tabelle where "
	if {$ID ne ""} {
		append Befehl "ID=$ID"
	} else {
		foreach Feld $ListeFelder {
			append Befehl "$Feld like \"$Suche\" or "
		}
		set Befehl [string range $Befehl 0 end-4]
	}
	append Befehl " order by "
	foreach Feld $ListboxFelder {
		append Befehl "$Feld || "
	}
	set Befehl [string range $Befehl 0 end-4]
	append Befehl " LIMIT [dict get $Glob ListboxLimit]"
	#puts $Befehl
	
	db1 eval "$Befehl" Ergebnis {
		#parray Ergebnis
		set Ausgabe ""
		for {set Zaehler 0} {$Zaehler < [llength $ListboxFelder]} {incr Zaehler} {
			set Feld [lindex $ListboxFelder $Zaehler]
			set Trennzeichen [lindex $ListboxTrennzeichen $Zaehler]
			if {$Ergebnis($Feld) ne ""} {
				append Ausgabe $Ergebnis($Feld)
				append Ausgabe $Trennzeichen
				set Laenge [string length $Trennzeichen]
			}			
		}
		set Ausgabe [string range $Ausgabe 0 end-$Laenge]
		lappend ListeEintrag $Ausgabe
		lappend ListeID $Ergebnis(ID)
	}
	dict set Rueckgabe Eintrag $ListeEintrag
	dict set Rueckgabe ID $ListeID
	return $Rueckgabe
}

proc FelderLeeren {} {
	# leert die Entry-Felder
	global Glob
	global GlobEntryInhalt

	set ListeFelder [dict get $Glob ListeFelder]
	set ListeHoehe [dict get $Glob ListeHoehe]

	set GlobEntryInhalt(ID) ""
	for {set Zaehler 0} {$Zaehler < [llength $ListeFelder]} {incr Zaehler} {
		set GlobEntryInhalt($Zaehler) ""
		set Hoehe [lindex $ListeHoehe $Zaehler]
		if {$Hoehe > 1} {
			set Status [.frRechts.c.frFelder.tx$Zaehler cget -state]
			.frRechts.c.frFelder.tx$Zaehler configure -state normal
			.frRechts.c.frFelder.tx$Zaehler delete 1.0 end
			.frRechts.c.frFelder.tx$Zaehler configure -state $Status
		}
	}	
}

proc btNeuAendern {} {
	# Button Neu
	# Schaltet die Entry-Felder zum Ändern frei bzw. leert die Felder für die Neuerfassung
	# Modus: 1=Neu 2=Ändern
	global Glob

	set ListeHoehe [dict get $Glob ListeHoehe]
	set ListeFelder [dict get $Glob ListeFelder]
	
	if {[dict get $Glob Modus] == 1} {
		FelderLeeren
	}
	
	# Status der Felder ändern
	for {set Zaehler 0} {$Zaehler < [llength $ListeFelder]} {incr Zaehler} {
		set Hoehe [lindex $ListeHoehe $Zaehler]
		if {$Hoehe == 1} {
			.frRechts.c.frFelder.en$Zaehler configure -state normal
		} else {
			.frRechts.c.frFelder.tx$Zaehler configure -state normal
		}
	}	
		
	# Status der Buttons ändern
	.frUnten.btSpeichern configure -state normal
	.frUnten.btAbbruch configure -state normal
	.frUnten.btAendern configure -state disabled
	.frUnten.btLoeschen configure -state disabled
	.frUnten.btExportieren configure -state disabled
	.frUnten.btImportieren configure -state disabled
	
	update idletasks
}

proc Anzeigen {ID} {
	# Zeigt die Details zum markierten Datensatz an
	# ID = ID des markierten Datensatzes
	global Glob

	set ListeHoehe [dict get $Glob ListeHoehe]
	set ListeFelder [dict get $Glob ListeFelder]
	
	if {$ID ne ""} {
		DatensatzAnzeigen $ID
		# Status der Felder ändern
		for {set Zaehler 0} {$Zaehler < [llength $ListeFelder]} {incr Zaehler} {
			set Hoehe [lindex $ListeHoehe $Zaehler]
			if {$Hoehe == 1} {
				.frRechts.c.frFelder.en$Zaehler configure -state readonly
			} else {
				.frRechts.c.frFelder.tx$Zaehler configure -state disabled
			}
		}	
		
		# Status der Buttons ändern
		.frUnten.btAbbruch configure -state disabled
		.frUnten.btSpeichern configure -state disabled
		.frUnten.btAendern configure -state normal
		.frUnten.btLoeschen configure -state normal
		.frUnten.btExportieren configure -state normal
		.frUnten.btImportieren configure -state normal

		update idletasks
	}
}

proc btSuchen {Suchbegriff} {
	# Button Suchen
	# Durchsucht die Datenbank nach passenden Einträgen
	# Suchbegriff = Suchbegriff
	global GlobListe

	set CursorAlt [. cget -cursor]
	. configure -cursor watch
	update idletasks
	
	.frLinks.frListe.lbox selection clear 0 end
	set Gefunden [Suchen $Suchbegriff]
	set GlobListe(ID) [dict get $Gefunden ID]
	set GlobListe(Eintrag) [dict get $Gefunden Eintrag]

	if {[llength $GlobListe(ID)] > 0} {
		.frLinks.frListe.lbox selection set 0
		.frUnten.btAendern configure -state normal
		.frUnten.btLoeschen configure -state normal
	} else {
		FelderLeeren
		.frUnten.btAendern configure -state disabled
		.frUnten.btLoeschen configure -state disabled
	}

	. configure -cursor $CursorAlt
	update idletasks
}

proc InhaltPruefen {Text} {
	# prüft den Inhalt der Eingabefelder und ersetzt unzulässige Zeichen
	regsub -all \" $Text "" Text
	regsub -all \t $Text " " Text
	regsub -all \n $Text " " Text
	regsub -all \r $Text " " Text
	regsub -all ";" $Text "." Text
	return $Text
}
proc btSpeichern {} {
	# Button Speichern
	# Speichert den neuen oder geänderten Datensatz
	global GlobEntryInhalt
	global Glob

	set ListeHoehe [dict get $Glob ListeHoehe]
	set ListeFelder [dict get $Glob ListeFelder]

	set ID ""
	if {[dict get $Glob Modus] == 1} {
		set Befehl "insert into Tabelle values(NULL,"
		for {set Zaehler 0} {$Zaehler < [llength $ListeFelder]} {incr Zaehler} {
			set Hoehe [lindex $ListeHoehe $Zaehler]
			if {$Hoehe > 1} {
				set GlobEntryInhalt($Zaehler) [.frRechts.c.frFelder.tx$Zaehler get 1.0 "end -1c"]
			}
			set Wert [InhaltPruefen $GlobEntryInhalt($Zaehler)]
			append Befehl "\"$Wert\","
		}
		set Befehl [string range $Befehl 0 end-1]
		append Befehl ")"
		db1 eval "$Befehl"
		db1 eval {select max(ID) as ID from Tabelle} Ergebnis {
			#parray Ergebnis
			set ID $Ergebnis(ID)
		}
	} else {
		set Befehl "update Tabelle set "
		for {set Zaehler 0} {$Zaehler < [llength $ListeFelder]} {incr Zaehler} {
			set Hoehe [lindex $ListeHoehe $Zaehler]
			if {$Hoehe > 1} {
				set GlobEntryInhalt($Zaehler) [.frRechts.c.frFelder.tx$Zaehler get 1.0 "end -1c"]
			}
			set Feld [lindex $ListeFelder $Zaehler]
			set Wert [InhaltPruefen $GlobEntryInhalt($Zaehler)]
			append Befehl "$Feld=\"$Wert\","
		}
		set Befehl [string range $Befehl 0 end-1]
		append Befehl " where ID=$GlobEntryInhalt(ID)"
		db1 eval "$Befehl"
		set ID $GlobEntryInhalt(ID)
	}
	return $ID
}

proc btLoeschen {ID} {
	# Button Löschen
	# Löscht den markierten Datensatz
	set Antwort [tk_messageBox -title [::msgcat::mc "Frage"] -message [::msgcat::mc "Wollen Sie den Datensatz wirklich loeschen?"] -icon question -type yesno -default no]
	if {$Antwort == yes} {
		db1 eval {delete from Tabelle where ID=$ID}
		FelderLeeren
	}
}

proc btExportieren {} {
	# Button Exportieren
	# Exportiert die Datenbank als csv-Datei
	global Glob

	#set Antwort [tk_messageBox -title [::msgcat::mc "Frage"] -message [::msgcat::mc "Sollen die Feldnamen als erste Zeile exportiert werden?"] -icon question -type yesno -default yes]
	set Antwort "yes"
	
	set ListeFelder [dict get $Glob ListeFelder]

	set tmp [dict get $Glob InitialOrdnerImportExport]
	set Dateiname [tk_getSaveFile -initialdir $tmp -defaultextension .csv]
	if {$Dateiname != ""} {
		set CursorAlt [. cget -cursor]
		. configure -cursor watch
		update idletasks
	
		set f [open $Dateiname w]
		
		if {$Antwort == "yes"} {
			set Datensatz "ID;"
			foreach  Feld $ListeFelder {
				append Datensatz $Feld
				append Datensatz ";"
			}
			puts $f $Datensatz
		}
		
		db1 eval {select * from Tabelle} Ergebnis {
			#parray Ergebnis
			set Datensatz ""
			append Datensatz $Ergebnis(ID)
			append Datensatz ";"
			foreach  Feld $ListeFelder {
				append Datensatz [string map {; .} $Ergebnis($Feld)]
				append Datensatz ";"
			}
			puts $f $Datensatz
		}
		close $f

		. configure -cursor $CursorAlt
		update idletasks
		
		tk_messageBox -title [::msgcat::mc "Info"] -message [::msgcat::mc "Die Daten wurden exportiert."] -icon info -type ok -default ok
	}
}

proc btImportieren {} {
	# Button Importieren
	# Importiert eine csv-Datei in die Datenbank
	# Die erste Spalte ID muss in der csv-Datei vorhanden sein, wird aber beim Import in die Datenbank ignoriert
	global Glob

	#set Antwort [tk_messageBox -title [::msgcat::mc "Frage"] -message [::msgcat::mc "Enthaelt die erste Zeile die Spaltenueberschrift?"] -icon question -type yesno -default no]
	set Antwort "no"

	set ListeFelder [dict get $Glob ListeFelder]
	
	set tmp [dict get $Glob InitialOrdnerImportExport]
	set Dateiname [tk_getOpenFile -initialdir $tmp -filetypes {{{csv} {*.csv}} {{Alle} {*}}}]
	if {$Dateiname != ""} {
		if {[glob -nocomplain -types f $Dateiname] eq ""} {return}
		set CursorAlt [. cget -cursor]
		. configure -cursor watch
		update idletasks

		set f [open $Dateiname r]
		if {$Antwort == "yes"} {set Zeile [gets $f]}
		db1 eval {begin transaction}
		while {[gets $f Zeile] >= 0} {
			set Befehl "insert into Tabelle values(NULL,"
			set Liste [split $Zeile ";"]
			for {set Zaehler 0} {$Zaehler < [llength $ListeFelder]} {incr Zaehler} {
				if {$Zaehler < [llength $Liste]} {
					set Element [lindex $Liste [expr $Zaehler + 1]]
				} else {
					set Element ""
				}
				append Befehl "\"$Element\","
			}
			set Befehl [string range $Befehl 0 end-1]
			append Befehl ")"
			db1 eval "$Befehl"
		}
		db1 eval {end transaction}
		close $f

		. configure -cursor $CursorAlt
		update idletasks
		
		tk_messageBox -title [::msgcat::mc "Info"] -message [::msgcat::mc "Die Datei wurde importiert."] -icon info -type ok -default ok
	}
}

proc SpracheEinstellen {} {
	# stellt die Sprache ein
	global Glob

	if {[dict get $Glob Sprache] eq "de" } {
		dict set Glob Sprache "en"
	} else {
		dict set Glob Sprache "de"
	}

	switch [dict get $Glob Sprache] {
		de {::msgcat::mclocale de}
		en {::msgcat::mclocale en}
	}
	set tmp [dict get $Glob Skriptname]
	::msgcat::mcload [file join [file dirname $tmp]]
	
	.frLinks.frSuche.bt configure -text [::msgcat::mc "Suchen"]
	.frLinks.frSuche.btAlle configure -text [::msgcat::mc "Alle"]
	.frUnten.btNeu configure -text [::msgcat::mc "Neu"]
	.frUnten.btAendern configure -text [::msgcat::mc "Aendern"]
	.frUnten.btLoeschen configure -text [::msgcat::mc "Loeschen"]
	.frUnten.btAbbruch configure -text [::msgcat::mc "Abbruch"]
	.frUnten.btSpeichern configure -text [::msgcat::mc "Speichern"]
	.frUnten.btExportieren configure -text [::msgcat::mc "Exportieren"]
	.frUnten.btImportieren configure -text [::msgcat::mc "Importieren"]
	.frUnten.btSprache configure -text [::msgcat::mc "English"]
	.frUnten.btInfo configure -text [::msgcat::mc "Info"]
	
	update idletasks
}

proc DateienPruefen {} {
	# prüft, ob alle Dateien vorhanden sind
	global Glob
	
	set tmp [dict get $Glob Tabelle]
	if {[glob -nocomplain -types f $tmp] eq ""} {set Antwort [tk_messageBox -title [::msgcat::mc "Fehler"] -message [::msgcat::mc "Die Datei %s existiert nicht." $tmp] -icon error -type ok -default ok];exit}
	
	set tmp [dict get $Glob ListboxAusgabe]
	if {[glob -nocomplain -types f $tmp] eq ""} {set Antwort [tk_messageBox -title [::msgcat::mc "Fehler"] -message [::msgcat::mc "Die Datei %s existiert nicht." $tmp] -icon error -type ok -default ok];exit}
}

proc InfoAnzeigen {ProgrammName ProgrammVersion} {
	set Titel [::msgcat::mc "Info"]
	set Text "${ProgrammName}\nVersion ${ProgrammVersion}\n\nCopyright (C) 2016 Oliver Scholl\nwww.tcltk.info\n
This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the license, or any later version.\n
This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of merchantability or fitness for a particular purpose. See the GNU General Public License for more details."
	tk_messageBox -title $Titel -message $Text -type ok
}

# *******************************************************************************
# Hauptprogramm

dict set Glob Skriptname [info script]
if {[file type [dict get $Glob Skriptname]] == "link"} {
	dict set Glob Skriptname [file readlink [dict get $Glob Skriptname]]
}
::msgcat::mclocale de ;# Alternative Sprachen: de=deutsch en=englisch
::msgcat::mcload [file join [file dirname [dict get $Glob Skriptname]]]

dict set Glob KonfigurationOrdner [file join $env(HOME) [dict get $Glob ProgrammOrdner] [dict get $Glob ProgrammName]]
if {[file exist [dict get $Glob KonfigurationOrdner]] == 0} {
	file mkdir [dict get $Glob KonfigurationOrdner]
}

dict set Glob KonfigurationDateiname [file join [dict get $Glob KonfigurationOrdner] Konf.txt] ; # Dateiname der Konfigurationsdatei

dict set Glob Sprache "en"
dict set Glob DatenbankOrdner ""
dict set Glob Fensterbreite 790
dict set Glob Fensterhoehe 590
dict set Glob FensterX 10
dict set Glob FensterY 10

KonfigurationLaden1

if {[dict get $Glob DatenbankOrdner] == ""} {
	dict set Glob DatenbankOrdner $env(HOME)
}
dict set Glob DatenbankOrdner [tk_chooseDirectory -initialdir [dict get $Glob DatenbankOrdner]]
if {[dict get $Glob DatenbankOrdner] eq ""} {exit}

dict set Glob KonfigurationFenster [file join [dict get $Glob DatenbankOrdner] Fenster.konf] ; # Fensterinformationen (Breite, Hoehe, Position). Wird im Datenbankordner gespeichert, weil die Größe vom Inhalt der DB abhängt.
dict set Glob InitialOrdnerImportExport $env(HOME) ; #Initialordner, wenn man den Button Export / Import anklickt
dict set Glob Datenbank [file join [dict get $Glob DatenbankOrdner] Datenbank.db] ; # Datenbankname
dict set Glob Tabelle [file join [dict get $Glob DatenbankOrdner] Tabelle.txt]
dict set Glob ListboxAusgabe [file join [dict get $Glob DatenbankOrdner] Liste.txt]

dict set Glob ListeFelder "" ; # Liste mit allen Feldern der Tabelle
dict set Glob ListeLabels "" ; # Liste mit allen Labels der Felder
dict set Glob ListeHoehe "" ; # Liste mit der Zeilenhöhe der Felder. Wenn 1, dann wird ein Entry-Widget verwendet. Wenn > 1, dann wird ein Text-Widget verwendet
dict set Glob ListboxFelder "" ; # Liste der Felder in der Listbox
dict set Glob ListboxTrennzeichen "" ; # Liste der Trennzeichen in der Listbox
dict set Glob ListboxLimit 100000 ; # maximale Anzahl der Listbox-Eintraege
dict set Glob Modus 1 ; #Unterscheidet zwischen Ändern und Neuanlage eines Datensatzes. 1=Neu 2=Ändern
dict set Glob NeuAendernID 0; # speichert die ID des Datensatzes, der geändert werden soll. Falls dann Abbruch angeklickt wird, wird dieser Datensatz wieder angezeigt
array set GlobListe {} ;	# Liste der Einträge in der Listbox. Es gibt:
							# GlobListe(Eintrag) = Texteintrag in der Listbox
							# GlobListe(ID) = Datensatz ID zum Eintrag in der Listbox
KonfigurationLaden2

DateienPruefen
FelderEinlesen
DatenbankOeffnen

# Sprache invertieren, weil der Button de/en seinerseits die Sprache invertiert
switch [dict get $Glob Sprache] {
	de {dict set Glob Sprache "en"}
	en {dict set Glob Sprache "de"}
}

set tmpOrdner [dict get $Glob DatenbankOrdner]
if {[string length $tmpOrdner] > 80} {
	set tmpOrdner [string range [dict get $Glob DatenbankOrdner] 0 20]
	append tmpOrdner "..."
	append tmpOrdner [string range [dict get $Glob DatenbankOrdner] end-40 end]
}
wm title . "[dict get $Glob ProgrammTitel] $tmpOrdner"
wm protocol . WM_DELETE_WINDOW {DatenbankSchliessen;KonfigurationSpeichern;exit}

ttk::frame .frLinks
ttk::frame .frLinks.frSuche
ttk::entry .frLinks.frSuche.en -textvariable Suchbegriff -width 30
ttk::button .frLinks.frSuche.bt -text [::msgcat::mc "Suchen"] -command {btSuchen $Suchbegriff;Anzeigen [lindex $GlobListe(ID) [.frLinks.frListe.lbox curselection]]}
ttk::button .frLinks.frSuche.btAlle -text [::msgcat::mc "Alle"] -command {set Suchbegriff "";.frLinks.frSuche.bt invoke}
pack .frLinks.frSuche.btAlle -side right -padx 2 -pady 2
pack .frLinks.frSuche.bt -side right -padx 2 -pady 2
pack .frLinks.frSuche.en -side left -padx 2 -pady 2 -expand yes -fill x

ttk::frame .frLinks.frListe
listbox .frLinks.frListe.lbox -listvariable GlobListe(Eintrag) -xscrollcommand ".frLinks.frListe.sbx set" -yscrollcommand ".frLinks.frListe.sby set" -selectmode single
ttk::scrollbar .frLinks.frListe.sbx -orient horizontal -command ".frLinks.frListe.lbox xview"
ttk::scrollbar .frLinks.frListe.sby -command ".frLinks.frListe.lbox yview"
pack .frLinks.frListe.sbx -side bottom -fill x
pack .frLinks.frListe.sby -side right -fill y
pack .frLinks.frListe.lbox -side top -expand yes -fill both

pack .frLinks.frSuche -side top -fill x
pack .frLinks.frListe -side bottom -expand yes -fill both

#***********************
ttk::frame .frRechts

canvas .frRechts.c -width 350 -height 300 -xscrollcommand ".frRechts.xscroll set" -yscrollcommand ".frRechts.yscroll set" ; # -scrollregion "0 0 350 400"
ttk::scrollbar .frRechts.xscroll -orient horizontal -command ".frRechts.c xview"
ttk::scrollbar .frRechts.yscroll -command ".frRechts.c yview"
pack .frRechts.xscroll -side bottom -fill x
pack .frRechts.yscroll -side right -fill y
pack .frRechts.c -expand yes -fill both -side top

ttk::frame .frRechts.c.frFelder
ttk::label .frRechts.c.frFelder.lbID -text [::msgcat::mc "ID:"]
ttk::entry .frRechts.c.frFelder.enID -textvariable GlobEntryInhalt(ID) -state disabled -width 10
grid .frRechts.c.frFelder.lbID -row 0 -column 0 -sticky w -padx 2 -pady 2
grid .frRechts.c.frFelder.enID -row 0 -column 1 -sticky w -padx 2 -pady 2

set ListeFelder [dict get $Glob ListeFelder]
set ListeLabels [dict get $Glob ListeLabels]
set ListeHoehe [dict get $Glob ListeHoehe]
set EntryBreite 40
for {set Zaehler 0} {$Zaehler < [llength $ListeFelder]} {incr Zaehler} {
	set Entry [lindex $ListeFelder $Zaehler]
	set Hoehe [lindex $ListeHoehe $Zaehler]
	set Label [lindex $ListeLabels $Zaehler]
	if {$Label eq ""} {
		set Label $Entry
	}
	append Label ":"
	ttk::label .frRechts.c.frFelder.lb$Zaehler -text $Label
	grid .frRechts.c.frFelder.lb$Zaehler -row [expr $Zaehler + 1] -column 0 -sticky nw -padx 2 -pady 2

	if {$Hoehe == 1} {
		ttk::entry .frRechts.c.frFelder.en$Zaehler -textvariable GlobEntryInhalt($Zaehler) -state disabled -width $EntryBreite
		grid .frRechts.c.frFelder.en$Zaehler -row [expr $Zaehler + 1] -column 1 -sticky w -padx 2 -pady 2
		bind .frRechts.c.frFelder.en$Zaehler <KeyPress-quotedbl> break
		bind .frRechts.c.frFelder.en$Zaehler <KeyPress-semicolon> break
	} else {
		text .frRechts.c.frFelder.tx$Zaehler -state disabled -width [expr $EntryBreite + 5] -height $Hoehe -wrap word
		grid .frRechts.c.frFelder.tx$Zaehler -row [expr $Zaehler + 1] -column 1 -sticky w -padx 2 -pady 2
		bind .frRechts.c.frFelder.tx$Zaehler <KeyPress-Return> break
		bind .frRechts.c.frFelder.tx$Zaehler <KeyPress-Tab> break
		bind .frRechts.c.frFelder.tx$Zaehler <KeyPress-quotedbl> break
		bind .frRechts.c.frFelder.tx$Zaehler <KeyPress-semicolon> break
	}

}
pack .frRechts.c.frFelder

.frRechts.c create window 2 2 -anchor nw -window .frRechts.c.frFelder
#.frRechts.c configure -scrollregion [.frRechts.c bbox all]
set ListeFelder [dict get $Glob ListeFelder]
set Breite [expr $EntryBreite*11]

set ListeHoehe [dict get $Glob ListeHoehe]
set Hoehe 1
for {set Zaehler 0} {$Zaehler < [llength $ListeFelder]} {incr Zaehler} {
	set tmpHoehe [lindex $ListeHoehe $Zaehler]							
	incr Hoehe $tmpHoehe
}
set Hoehe [expr $Hoehe * 27]

.frRechts.c configure -scrollregion "0 0 $Breite $Hoehe"
.frRechts.c configure -width $Breite
#**********************

ttk::frame .frUnten
ttk::button .frUnten.btNeu -text [::msgcat::mc "Neu"] -command {dict set Glob Modus 1; btNeuAendern}
ttk::button .frUnten.btAendern -text [::msgcat::mc "Aendern"] -command {dict set Glob Modus 2;dict set Glob NeuAendernID [lindex $GlobListe(ID) [.frLinks.frListe.lbox curselection]]; btNeuAendern} -state disabled
ttk::button .frUnten.btLoeschen -text [::msgcat::mc "Loeschen"] -command {btLoeschen [lindex $GlobListe(ID) [.frLinks.frListe.lbox curselection]];set Suchbegriff "";.frLinks.frSuche.bt invoke} -state disabled
ttk::button .frUnten.btAbbruch -text [::msgcat::mc "Abbruch"] -command {set ID [dict get $Glob NeuAendernID];set Suchbegriff "ID=$ID";.frLinks.frSuche.bt invoke} -state disabled
ttk::button .frUnten.btSpeichern -text [::msgcat::mc "Speichern"] -command {set ID [btSpeichern];set Suchbegriff "ID=$ID";.frLinks.frSuche.bt invoke} -state disabled
ttk::button .frUnten.btExportieren -text [::msgcat::mc "Exportieren"] -command {btExportieren}
ttk::button .frUnten.btImportieren -text [::msgcat::mc "Importieren"] -command {btImportieren;set Suchbegriff "";.frLinks.frSuche.bt invoke}
ttk::button .frUnten.btSprache -text [::msgcat::mc "English"] -command {SpracheEinstellen}
ttk::button .frUnten.btInfo -text [::msgcat::mc "Info"] -command {InfoAnzeigen [dict get $Glob ProgrammName] [dict get $Glob ProgrammVersion]}
pack .frUnten.btNeu -side left
pack .frUnten.btAendern -side left
pack .frUnten.btLoeschen -side left
pack .frUnten.btAbbruch -side left
pack .frUnten.btSpeichern -side left
pack .frUnten.btExportieren -side left
pack .frUnten.btImportieren -side left
pack .frUnten.btSprache -side left
pack .frUnten.btInfo -side left

grid .frLinks -row 0 -column 0 -sticky nesw  -padx 2 -pady 2
grid .frRechts -row 0 -column 1 -sticky nesw -padx 2 -pady 2
grid .frUnten -row 1 -column 0 -columnspan 2 -padx 2 -pady 2
grid rowconfigure . 0 -weight 1
grid columnconfigure . 0 -weight 1
grid columnconfigure . 1 -weight 0

bind .frLinks.frSuche.en <KeyPress-quotedbl> break
bind .frLinks.frSuche.en <KeyRelease-Return> {.frLinks.frSuche.bt invoke}
bind .frLinks.frListe.lbox <ButtonRelease-1> {Anzeigen [lindex $GlobListe(ID) [.frLinks.frListe.lbox curselection]]}
bind .frLinks.frListe.lbox <KeyRelease-Down> {set Index [.frLinks.frListe.lbox curselection];.frLinks.frListe.lbox selection clear 0 end; incr Index; if {$Index == [llength $GlobListe(ID)]} {incr Index -1}; .frLinks.frListe.lbox selection set $Index;Anzeigen [lindex $GlobListe(ID) [.frLinks.frListe.lbox curselection]]}
bind .frLinks.frListe.lbox <KeyRelease-Up> {set Index [.frLinks.frListe.lbox curselection];.frLinks.frListe.lbox selection clear 0 end; if {$Index > 0} {incr Index -1}; .frLinks.frListe.lbox selection set $Index;Anzeigen [lindex $GlobListe(ID) [.frLinks.frListe.lbox curselection]]}

set tmpBreite [dict get $Glob Fensterbreite]
set tmpHoehe [dict get $Glob Fensterhoehe]
set tmpX [dict get $Glob FensterX]
set tmpY [dict get $Glob FensterY]
wm geometry . ${tmpBreite}x${tmpHoehe}+${tmpX}+${tmpY}

SpracheEinstellen

set Suchbegriff ""
.frLinks.frSuche.bt invoke
focus .frLinks.frListe.lbox
